﻿using OpcLabs.BaseLib.ComponentModel.Internal;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.BaseLib.Graphs;
using OpcLabs.BaseLib.Graphs.Internal;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Forms;

namespace OpcLabs.BaseLibFormsTestApp
{
    public partial class TraverserTestForm : Form
    {
        public TraverserTestForm()
        {
            InitializeComponent();
        }

        private void showButton_Click(object sender, EventArgs e)
        {
            //TypeDescriptor.AddAttributes(typeof(IList), new TypeConverterAttribute(typeof(ExpandableCollectionConverter))); // global for PropertyGrid

            var item1 = new Item() { ItemName = "Item1", Content = "c1" };
            var item2 = new Item() { ItemName = "Item2", Content = "c2" };
            var item3 = new Item() { ItemName = "Item3", Content = "c3" };
            var user3 = new User() { Name = "Name1", Old = 5, Items = new List<Item> { item1, item2, item3 } };
            var user2 = new User() { Name = "Name1", Old = 5, Items = new List<Item> { item1, item2, item3 }, Parent = user3 };
            var user1 = new User() { Name = "Name1", Old = 5, Items = new List<Item> { item1, item2, item3 }, ItemArray = new Item[] { item1, item2, item3 }, Parent = user2, TraversalType = TreeTraversalType.BreadthFirst };
            user1.KeyedItems.Add("first", item1);

            //var data1 = new SequenceData(new GenericData [] { "first", "second", "thrid" });
            //var data2 = new SequenceData(new object[] { "first", "second", data1, "xxx", "iko", "kjh", "jkiuh", "jhygt", "kk", "mm", "jh", "ss", "rr" });


            propertyGrid1.SelectedObject = user1; // user1;


            var attributeStore = new AttributeStore();
            attributeStore.AddAttributesForType(typeof(ItemDictionary), true, new TypeConverterAttribute(typeof(ExpandableDictionaryConverter))); // expand ItemDictionary type
            attributeStore.AddAttributesForInstance(user1, false, nameof(User.Items), new TypeConverterAttribute(typeof(ExpandableCollectionConverter))); // expand Items property only for user1 instance 
            //attributeStore.AddAttributesForType(typeof(IList), true, new TypeConverterAttribute(typeof(ExpandableCollectionConverter))); // all collection expandable
            attributeStore.AddAttributesForInstance(item2, false, new TypeConverterAttribute(typeof(TypeConverter))); // do not expand item2 instance

            var traversable = new PropertyForestTraversable(user1, attributeStore);
            // or
            // var traversable = new PropertyForestTraversable<PropertyDescriptor>(user1, Filter, attributeStore);

            foreach (var item in traversable.AddIndexes().ToEnumerable())
            {
                var line = new string(' ', 4 * item.Key.Length) + item.Value.PropertyDescriptor.DisplayName + ": " + (item.Value.GetValue() ?? "(null)");
                richTextBox1.AppendText(line + Environment.NewLine);
            }


        }

        private PropertyDescriptor Filter(PropertyDescriptor property)
        {
            return property;
            // only Browsable
            if (property.IsBrowsable) return property; 
            return null;
        }

    }

    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class User
    {
        public string Name { get; set; }

        [ReadOnly(true)]
        public int Old { get; set; }

        public Item this[int index] { get => Items[index]; set => Items[index] = value; }

        public List<Item> Items { get; set; }

        [DisplayName("Item Array")]
        public Item[] ItemArray { get; set; }

        public User Parent { get; set; }

        public object SomeInfo { get; set; }

        //[TypeConverter(typeof(ExpandableDictionaryConverter))]
        public ItemDictionary KeyedItems { get; } = new ItemDictionary();

        public TreeTraversalType TraversalType { get; set; }
    }

    //    [TypeDescriptionProvider(typeof(TypeDescriptionProvider<Dictionary<string, Item>, DictionaryTypeDescriptor>))]
    //[TypeDescriptionProvider(typeof(TypeDescriptionProvider<ItemDictionary, ExpandableDictionaryTypeDescriptor>))]
    public class ItemDictionary : Dictionary<string, Item>
    { }


    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class Item
    {
        public string ItemName { get; set; }

        public object Content { get; set; }
    }

}
