﻿using System.Collections.Generic;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Identification;
using OpcLabs.UnitTestLib;

namespace OpcLabs.BaseLibTest.Identification
{
    public /*partial*/ class CookieManagerTest
    {
        static private void UniquenessAndValidityTest([NotNull] CookieManager target, int count, int displayEach)
        {
            var cookieSet = new HashSet<int>();
            for (int i = 0; i < count; i++)
            {
                int cookie = target.NextCookieValue();
                if ((i % displayEach) == 0)
                    AssertUtilities.WriteLine(cookie);
                Assert.IsTrue(CookieManager.IsValidCookieValue(cookie), "invalid");
                Assert.IsFalse(cookieSet.Contains(cookie), "not unique");
                cookieSet.Add(cookie);
            }
        }
        static private void UniquenessAndValidityTest(int count, int displayEach)
        {
            var target = CookieManager.Create();
            UniquenessAndValidityTest(target, count, displayEach);
        }

        [TestMethod]
        // ReSharper disable InconsistentNaming
        public void UniquenessAndValidityTest_20()
        // ReSharper restore InconsistentNaming
        {
            UniquenessAndValidityTest(20, 1);
        }

        [TestMethod]
        // ReSharper disable InconsistentNaming
        public void UniquenessAndValidityTest_20_Offset()
        // ReSharper restore InconsistentNaming
        {
            var target = CookieManager.Create(777777777, shared:false);
            UniquenessAndValidityTest(target, 20, 1);
        }

        [TestMethod]
        // ReSharper disable InconsistentNaming
        public void UniquenessAndValidityTest_20_WrapOverInvalid()
        // ReSharper restore InconsistentNaming
        {
            var target = CookieManager.Create(CookieManager.InvalidCookie - 10, shared:false);
            UniquenessAndValidityTest(target, 20, 1);
        }

        [TestMethod]
        // ReSharper disable InconsistentNaming
        public void UniquenessAndValidityTest_100000()  // be over 16-bit size (65536)
        // ReSharper restore InconsistentNaming
        {
            UniquenessAndValidityTest(100000, 100);
        }
    }
}
