﻿using System;
using System.Runtime.InteropServices;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.Extensions;
using OpcLabs.EasyOpc.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.DataAccess.Extensions
{
    // ReSharper disable once InconsistentNaming
    public abstract class IEasyDAClientExtension2TestBase
    {
        #region Test Methods

        [TestMethod]
        public void WaitForItemValueTest_Constant()
        {
            TestSection.Arrange();

            var client = new EasyDAClient();

            TestSection.Act();

            object value;
            try
            {
                value = client.WaitForItemValue(
                    DALiveTestEnvironment.OpcLabs_KitServer.ServerDescriptors.WithProgId,
                    "Simulation.Constant_I2",
                    groupParameters: 100,
                    millisecondsTimeout: 5 * 1000);
            }
            catch (OpcException opcException)
            {
                if (!client.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                throw;
            }
            AssertUtilities.WriteLine($"{nameof(value)}: {value}");

            TestSection.Assert();

            Assert.IsNotNull(value);
            Assert.IsInstanceOfType(value, typeof(Int16));
        }

        [TestMethod]
        public void WaitForItemValueTest_AlternatingError()
        {
            TestSection.Arrange();

            var client = new EasyDAClient();

            TestSection.Act();

            object value;
            try
            {
                value = client.WaitForItemValue(
                    DALiveTestEnvironment.OpcLabs_KitServer.ServerDescriptors.WithProgId,
                    "Simulation.AlternatingError (1 s)",
                    groupParameters: 100,
                    millisecondsTimeout: 10 * 1000);
            }
            catch (OpcException opcException)
            {
                AssertUtilities.WriteLine($"{nameof(opcException)}: {opcException}");
                if (!client.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                Exception innerException = opcException.InnerException;
                Assert.IsInstanceOfType(innerException, typeof(COMException));
                return;
            }
            AssertUtilities.WriteLine($"{nameof(value)}: {value}");

            TestSection.Assert();

            Assert.IsNotNull(value);
            Assert.IsInstanceOfType(value, typeof(Int32));
        }

        [TestMethod]
        public void WaitForItemValueTest_AlternatingQuality_Uncertain()
        {
            TestSection.Arrange();

            var client = new EasyDAClient();

            TestSection.Act();

            object value;
            try
            {
                value = client.WaitForItemValue(
                    DALiveTestEnvironment.OpcLabs_KitServer.ServerDescriptors.WithProgId,
                    "Simulation.AlternatingQuality Uncertain (1 s)",
                    groupParameters: 100,
                    millisecondsTimeout: 10 * 1000);
            }
            catch (OpcException opcException)
            {
                if (!client.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                throw;
            }
            AssertUtilities.WriteLine($"{nameof(value)}: {value}");

            TestSection.Assert();

            Assert.IsNotNull(value);
            Assert.IsInstanceOfType(value, typeof(Int32));
        }

        [TestMethod]
        public void WaitForItemValueTest_Error()
        {
            TestSection.Arrange();

            var client = new EasyDAClient();

            TestSection.Act();

            try
            {
                client.WaitForItemValue(
                    DALiveTestEnvironment.OpcLabs_KitServer.ServerDescriptors.WithProgId,
                    "UnknownItem",
                    groupParameters: 100,
                    millisecondsTimeout: 10 * 1000);
            }
            catch (OpcException opcException)
            {
                AssertUtilities.WriteLine($"{nameof(opcException)}: {opcException}");
                if (!client.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                Exception innerException = opcException.InnerException;
                Assert.IsInstanceOfType(innerException, typeof(COMException));
                return;
            }
            Assert.Fail();
        }

        #endregion



        #region Protected Properties

        [NotNull]
        protected DALiveTestEnvironment TestEnvironment => DALiveTestEnvironment.Current;

        #endregion

        [NotNull]
        protected abstract IEasyDAClient CreateClient();
    }
}
