﻿
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.AddressSpace;
using OpcLabs.EasyOpc.DataAccess.Extensions;
using OpcLabs.EasyOpc.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    partial class EasyDAClientTestBase
    {
        protected void BrowseNodesTest_Basic(
            [NotNull] ServerDescriptor serverDescriptor, 
            [NotNull] DANodeDescriptor parentNodeDescriptor,
            int expectedCount)  // -1 for "unknown"
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckServer(serverDescriptor);
            EasyDAClient client = CreateClient();

            TestSection.Act();
            DANodeElementCollection nodeElements;
            try
            {
                nodeElements = client.BrowseNodes(
                    serverDescriptor,
                    parentNodeDescriptor,
                    DABrowseParameters.Empty);
            }
            catch (OpcException opcException)
            {
                if (!client.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                throw;
            }

            TestSection.Assert();
            Assert.IsNotNull(nodeElements);
            AssertUtilities.WriteLine(nodeElements.Count);
            CollectionAssert.AllItemsAreNotNull(nodeElements);
            foreach (DANodeElement nodeElement in nodeElements)
                AssertUtilities.WriteLine($"nodeElement: {nodeElement}");
            if (expectedCount >= 0)
                Assert.AreEqual(expectedCount, nodeElements.Count);
        }

        protected void BrowsePropertiesTest_UnknownItemId([NotNull] ServerDescriptor serverDescriptor)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckServer(serverDescriptor);
            EasyDAClient client = CreateClient();

            DAPropertyElementCollection actualResultCollection;
            try
            {
                TestSection.Act();
                try
                {
                    actualResultCollection = client.BrowseProperties(serverDescriptor, "unknownItemId");
                }
                finally
                {
                    TestSection.Assert();
                }
            }
            catch (OpcException opcException)
            {
                if (!client.SupportsTechnology(serverDescriptor.Technology))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                Exception innerException = opcException.InnerException;
                Assert.IsNotNull(innerException);
                AssertBaseException(innerException,
                    -1073479673, // OPC_E_UNKNOWNITEMID
                    "http://opcfoundation.org/DataAccess/",
                    "E_UNKNOWN_ITEM_NAME");
                return;
            }
            AssertUtilities.WriteLine(actualResultCollection.Count);
            Assert.Fail();
        }
    }
}
