﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Diagnostics.Utilities;
using OpcLabs.BaseLib.Extensions.Internal;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.AddressSpace;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public partial class IEasyDAClientTest_Com
    {
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Idra_RuntimeDA_ArrayOfI4()
        {
            const string serverClass = "Idra.runtimeDA.2";
            var serverDescriptor = new ServerDescriptor(serverClass);
            if (!serverDescriptor.CanInstantiate())
                Assert.Inconclusive();

            var client = CreateClient();

            // Array of VT_I4
            object value = client.ReadItemValue(serverDescriptor, "InjectPC.IdraSystem.Data.ExtraParameters1");

            Assert.IsNotNull(value);
            Assert.IsInstanceOfType(value, typeof(Int32[]));
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Idra_RuntimeDA_ArrayOfR4()
        {
            const string serverClass = "Idra.runtimeDA.2";
            var serverDescriptor = new ServerDescriptor(serverClass);
            if (!serverDescriptor.CanInstantiate())
                Assert.Inconclusive();

            var client = CreateClient();

            // Array of VT_R4, was error
            object value = client.ReadItemValue(serverDescriptor, "InjectPC.IdraSystem.Data.ExtraParameters");

            Assert.IsNotNull(value);
            Assert.IsInstanceOfType(value, typeof(Single[]));
        }

    // The server should return 1 branch ("Static") under the root when started. After 1 minute, it should return an additional "Dynamic" branch.
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void Endress_RuntimeNewItems_BrowseNodes()
        {
            const string serverClass = "Endress.RuntimeNewItems.Test";
            //const string serverClass = "OpcTestLab.OpcAnalyzer";
            var serverDescriptor = new ServerDescriptor(serverClass);
            if (!serverDescriptor.CanInstantiate())
                Assert.Inconclusive();

            // Make sure the server's process is not running at this point
            ProcessUtilities.KillProcesses("DANSrvNet4");

            var client = CreateClient();
            // keep the client connected by creating a subscription
            client.SubscribeItem("", serverClass, "Static.Readable_1", 1000);

            var nodeFilter = new DABrowseParameters {BrowseFilter = DABrowseFilter.All};

            DANodeElementCollection nodes1 = client.BrowseNodes(serverDescriptor, "", nodeFilter);
            Assert.IsNotNull(nodes1);
            Assert.AreEqual(1, nodes1.Count);

            AssertUtilities.WaitingPhase(66*1000);

            DANodeElementCollection nodes2 = client.BrowseNodes(serverDescriptor, "", nodeFilter);
            Assert.IsNotNull(nodes2);
            Assert.AreEqual(2, nodes2.Count);
        }
    }
}
