﻿using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.AlarmsAndEvents;
using OpcLabs.EasyOpc.AlarmsAndEvents.AddressSpace;
using OpcLabs.EasyOpc.AlarmsAndEvents.OperationModel;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.AlarmsAndEvents
{
    public partial class EasyAEClientTest
    {
        // ReSharper disable InconsistentNaming

        [TestCategory("Live")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestMethod]
        public void BrowseNodesTest()
        {
            var target = new EasyAEClient();

            AENodeElementCollection actualResultCollection = target.BrowseNodes(
                new ServerDescriptor("", "OPCLabs.KitEventServer.2"),
                new AENodeDescriptor(""),
                new AEBrowseParameters(AEBrowseFilter.Areas));

            Assert.IsNotNull(actualResultCollection);
            int count = actualResultCollection.Count;
            Assert.AreEqual(1, count);

            //TODO:
        }

        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void PullNotificationTest_SubscribeEvents()
        {
            var client = new EasyAEClient { PullNotificationQueueCapacity = 1000 };

            int handle = client.SubscribeEvents("", "OPCLabs.KitEventServer.2", 1000);

            int count = 0;
            int startTickCount = System.Environment.TickCount;
            do
            {
                EasyAENotificationEventArgs eventArgs = client.PullNotification(millisecondsTimeout: 2 * 1000);
                if (eventArgs == null)
                    continue;
                AssertUtilities.WriteLine(eventArgs);
                count++;

                Assert.IsNotNull(eventArgs);
                Exception exception = eventArgs.Exception;
                Assert.IsNull(exception);
                // IMPROVE: assert more about the event
            } while (System.Environment.TickCount < startTickCount + 10 * 1000);

            client.UnsubscribeEvents(handle);

            AssertUtilities.WriteLine(count);
            Assert.IsTrue(count >= 1);
        }

        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void SubscribeEventsTest()
        {
            List<EasyAENotificationEventArgs> eventArgsList;
            using (var client = new EasyAEClient())
            {
                eventArgsList = new List<EasyAENotificationEventArgs>();

                AssertUtilities.Phase("Subscribing");
                int handle = client.SubscribeEvents("", "OPCLabs.KitEventServer.2", 1000, (sender, args) =>
                {
                    AssertUtilities.WriteLine(args);
                    eventArgsList.Add(args);
                });
                AssertUtilities.WaitingPhase(15 * 1000);
                AssertUtilities.Phase("Unsubscribing");
                client.UnsubscribeEvents(handle);
                AssertUtilities.WaitingPhase(5 * 1000);
            }

            int count = eventArgsList.Count;
            AssertUtilities.WriteLine($"Count: {count}");
            Assert.IsTrue(count >= 1, "No event notifications");

            EasyAENotificationEventArgs[] eventArgsArray = eventArgsList.ToArray();

            for (int i = 0; i < eventArgsArray.Length; i++)
            {
                AssertUtilities.WriteLine(i);
                EasyAENotificationEventArgs eventArgs = eventArgsArray[i];
                Assert.IsNotNull(eventArgs);
                AssertUtilities.WriteLine(eventArgs);
                Exception exception = eventArgs.Exception;
                Assert.IsNull(exception);
                // IMPROVE: assert more about the event
            }
            AssertUtilities.Done();
        }

        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void SubscribeEventsTest_AdvEventServer()
        {
            const string objectId = "AdvOPCServer.AdvEventServer.1";
            LegacyTestEnvironment.Current.Precheck(objectId);

            var client = new EasyAEClient();
            var eventArgsList = new List<EasyAENotificationEventArgs>();
            int handle = client.SubscribeEvents("", objectId, 1000,
                (sender, args) => eventArgsList.Add(args));
            AssertUtilities.WaitingPhase(20 * 1000);
            client.UnsubscribeEvents(handle);

            int count = eventArgsList.Count;
            AssertUtilities.WriteLine(count);
            Assert.IsTrue(count >= 1);

            EasyAENotificationEventArgs[] eventArgsArray = eventArgsList.ToArray();

            for (int i = 0; i < eventArgsArray.Length; i++)
            {
                AssertUtilities.WriteLine(i);
                EasyAENotificationEventArgs eventArgs = eventArgsArray[i];
                Assert.IsNotNull(eventArgs);
                Exception exception = eventArgs.Exception;
                Assert.IsNull(exception);
                // IMPROVE: assert more about the event
            }
        }

        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void SubscribeEventsTest_FilterByCategories()
        {
            const long categoryId = 15531778;

            var client = new EasyAEClient();
            var daClient = new EasyDAClient();
            var filter = new AESubscriptionFilter
            {
                Categories = new[] { categoryId }
            };
            var eventArgsList = new List<EasyAENotificationEventArgs>();
            int handle = client.SubscribeEvents(
                // ReSharper disable RedundantArgumentName
                serverDescriptor: new ServerDescriptor("", "OPCLabs.KitEventServer.2"),
                subscriptionParameters: new AESubscriptionParameters(1000, filter),
                refreshWhenActive: true,
                callback: (sender, args) =>
                {
                    AssertUtilities.WriteLine(args);
                    eventArgsList.Add(args);
                },
                state: null);
            // ReSharper restore RedundantArgumentName

            // Set some events to active state.
            daClient.WriteItemValue("", "OPCLabs.KitServer.2", "SimulateEvents.ConditionState1.Activate", true);
            daClient.WriteItemValue("", "OPCLabs.KitServer.2", "SimulateEvents.ConditionState2.Activate", true);

            AssertUtilities.WaitingPhase(10 * 1000);
            client.UnsubscribeEvents(handle);

            int count = eventArgsList.Count;
            AssertUtilities.WriteLine(count);
            Assert.IsTrue(count >= 1);

            EasyAENotificationEventArgs[] eventArgsArray = eventArgsList.ToArray();

            for (int i = 0; i < eventArgsArray.Length; i++)
            {
                AssertUtilities.WriteLine(i);
                EasyAENotificationEventArgs eventArgs = eventArgsArray[i];
                Assert.IsNotNull(eventArgs);
                AssertUtilities.WriteLine(eventArgs);
                Exception exception = eventArgs.Exception;
                Assert.IsNull(exception);
                AEEventData eventData = eventArgs.EventData;
                if (eventData != null)
                {
                    Assert.AreEqual(categoryId, eventData.CategoryId);
                    //...
                }
                // IMPROVE: assert more about the event
            }
        }

        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void SubscribeEventsTest_Failure_1()
        {
            // An event must be generated even if we connect to a wrong server - this used to be a problem.
            var client = new EasyAEClient();
            var eventArgsList = new List<EasyAENotificationEventArgs>();
            int handle = client.SubscribeEvents("", "Wrong.Server", 1000, (sender, args) => eventArgsList.Add(args));
            AssertUtilities.WaitingPhase(10 * 1000);
            client.UnsubscribeEvents(handle);

            int count = eventArgsList.Count;
            AssertUtilities.WriteLine(count);
            Assert.AreEqual(1, count);
            EasyAENotificationEventArgs eventArgs = eventArgsList[0];
            Assert.IsNotNull(eventArgs);
            Exception exception = eventArgs.Exception;
            Assert.IsNotNull(exception);
            AssertUtilities.WriteLine(exception);
            // IMPROVE: assert more about the event
        }

        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void SubscribeEventsTest_Failure_2()
        {
            // An event must be generated even if we connect to a wrong server - this used to be a problem.
            var client = new EasyAEClient();
            var eventArgsList = new List<EasyAENotificationEventArgs>();
            int handle = client.SubscribeEvents("", "OPCLabs.KitServer.2", 1000, (sender, args) => eventArgsList.Add(args));
            AssertUtilities.WaitingPhase(10 * 1000);
            client.UnsubscribeEvents(handle);

            int count = eventArgsList.Count;
            AssertUtilities.WriteLine(count);
            Assert.AreEqual(1, count);
            EasyAENotificationEventArgs eventArgs = eventArgsList[0];
            Assert.IsNotNull(eventArgs);
            Exception exception = eventArgs.Exception;
            Assert.IsNotNull(exception);
            AssertUtilities.WriteLine(exception);
            // IMPROVE: assert more about the event
        }

        // ReSharper restore InconsistentNaming
    }
}
