﻿using System;
using System.Collections.Generic;
using System.Net.NetworkInformation;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;
using OpcLabs.BaseLib.Collections.Generic.Extensions;
using OpcLabs.BaseLib.Net.NetworkInformation;
using OpcLabs.BaseLib.Net.Sockets;
using OpcLabs.EasyOpcLiveTest.UA;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.UA
{
    public class LegacyUATestEnvironment
    {
        #region Private Constructors

        private LegacyUATestEnvironment()
        {
        }

        #endregion



        #region Static Public Fields

        [NotNull]
        static public readonly LegacyUATestEnvironment Current = new LegacyUATestEnvironment();

        [NotNull]
        // ReSharper disable once InconsistentNaming
        static public OpcFoundation_UASampleServer OpcFoundation_SampleServer = new OpcFoundation_UASampleServer(UALiveTestDefinitions.HostNames.OpcUATest);

        [NotNull]
        // ReSharper disable once InconsistentNaming
        static public Softing_OpcUaNetDemoServer Softing_OpcUaNetDemoServer = new Softing_OpcUaNetDemoServer(UALiveTestDefinitions.HostNames.OpcUATest);

        #endregion



        #region Public Methods

        public void PingPrecheck([NotNull] EndpointDescriptor endpointDescriptor)
        {
            AssertUtilities.Phase($"Ping pre-checking endpoint \"{endpointDescriptor}\"");

            string message;
            lock (_pingPrecheckCache)
            {
                if (!_pingPrecheckCache.TryGetValue(endpointDescriptor, out message))
                {
                    message = InternalPingPrecheck(endpointDescriptor);
                    _pingPrecheckCache.AddNew(endpointDescriptor, message);
                }
            }
            if (message != null)
                Assert.Inconclusive(message);
        }

        public void PrecheckEndpoint([NotNull] EndpointDescriptor endpointDescriptor)
        {
            AssertUtilities.Phase($"Pre-checking endpoint \"{endpointDescriptor}\"");

            string message;
            lock (_precheckCache)
            {
                if (!_precheckCache.TryGetValue(endpointDescriptor, out message))
                {
                    message = InternalPrecheck(endpointDescriptor);
                    _precheckCache.AddNew(endpointDescriptor, message);
                }
            }
            if (message != null)
                Assert.Inconclusive(message);
        }

        #endregion



        #region Protected Methods

        protected string InternalPingPrecheck([NotNull] EndpointDescriptor endpointDescriptor)
        {
            string host = endpointDescriptor.Host ?? "";

            bool success;
            PingReply pingReply = null;
            try
            {
                pingReply = PingUtilities.Send(host);
                success = true;
            }
            catch (NotSupportedException)
            {
                // address is an IPv6 address and the local computer is running an operating system earlier than Windows 2000. 
                success = false;
            }
            catch (PingException)
            {
                // An exception was thrown while sending or receiving the ICMP messages. See the inner exception for the exact 
                // exception that was thrown.
                success = false;
            }

            if (!success || (pingReply.Status != IPStatus.Success))
                return $"Failed pre-check: Ping test failure ({host}).";
            return null;
        }

        [CanBeNull]
        protected string InternalPrecheck([NotNull] EndpointDescriptor endpointDescriptor)
        {
            // ReSharper disable once ConvertIfStatementToReturnStatement
            if (!TcpClientUtilities.CanCreate(endpointDescriptor))
                return $"Failed pre-check: Socket exception (\"{endpointDescriptor.Host}\", {endpointDescriptor.Port}).";
            return null;
        }

        #endregion



        #region Private Fields

        [NotNull]
        private readonly Dictionary<EndpointDescriptor, string> _pingPrecheckCache = new Dictionary<EndpointDescriptor, string>();

        [NotNull]
        private readonly Dictionary<EndpointDescriptor, string> _precheckCache = new Dictionary<EndpointDescriptor, string>();

        #endregion
    }
}
