﻿using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Specialized;
using OpcLabs.BaseLib.Networking;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.PubSub;
using OpcLabs.EasyOpc.UA.PubSub.Configuration;
using OpcLabs.EasyOpc.UA.PubSub.Configuration.Extensions;
using OpcLabs.EasyOpc.UA.PubSub.Configuration.Implementation;
using OpcLabs.EasyOpc.UA.PubSub.Engine;
using OpcLabs.EasyOpc.UA.PubSub.InformationModel;
using OpcLabs.EasyOpc.UA.PubSub.InformationModel.Extensions;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA.PubSub.InformationModel
{
    [TestClass]
    public class UAReadOnlyPubSubClientConfigurationLiveTest
    {
        #region Test Methods

        [TestMethod]
        public void FindPublishedDataSetElementTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            UAPublishedDataSetElement publishedDataSetElement = configuration.FindPublishedDataSetElement("Simple");

            TestSection.Assert();
            Assert.IsNotNull(publishedDataSetElement);
            Assert.AreEqual(Guid.Empty, publishedDataSetElement.DataSetClassId);

            Assert.AreEqual("Simple", publishedDataSetElement.Name);
            Assert.AreEqual(0, publishedDataSetElement.ExtensionFieldValueDictionary.Count);

            UADataSetMetaData dataSetMetaData = publishedDataSetElement.DataSetMetaData;
            Assert.AreEqual(Guid.Empty, dataSetMetaData.DataSetClassId);
            Assert.AreEqual("", dataSetMetaData.Description);
            Assert.IsFalse(dataSetMetaData.HasPromotedFields);
            Assert.AreEqual("Simple", dataSetMetaData.Name);
            Assert.AreEqual(0, dataSetMetaData.PromotedFieldCount);
            Assert.AreEqual(4, dataSetMetaData.FieldCount);

            UAFieldMetaData fieldMetaData0 = dataSetMetaData.Fields[0];
            Assert.IsNotNull(fieldMetaData0);
            Assert.AreEqual("BooleanValue", fieldMetaData0.Name);
            Assert.AreEqual(UABuiltInType.Boolean, fieldMetaData0.BuiltInType);
            Assert.AreEqual(UADataTypeIds.Boolean, fieldMetaData0.DataTypeId);
            Assert.AreEqual(Guid.Parse("e7b24eb6-d5fb-404b-a3e1-ce9a64885bd6"), fieldMetaData0.DataSetFieldId);
            Assert.AreEqual(0, fieldMetaData0.ArrayDimensions.Length);
            Assert.AreEqual(UADataSetFieldFlags.None, fieldMetaData0.DataSetFieldFlags);
            Assert.AreEqual("", fieldMetaData0.Description);
            Assert.AreEqual(0, fieldMetaData0.MaximumStringLength);
            Assert.AreEqual(-1, fieldMetaData0.ValueRank);

            UAFieldMetaData fieldMetaData1 = dataSetMetaData.Fields[1];
            Assert.IsNotNull(fieldMetaData1);
            Assert.AreEqual("Scalar.Int32.X", fieldMetaData1.Name);
            Assert.AreEqual(UABuiltInType.Int32, fieldMetaData1.BuiltInType);
            Assert.AreEqual(UADataTypeIds.Int32, fieldMetaData1.DataTypeId);
            Assert.AreEqual(Guid.Parse("ab5125ad-517d-41ec-926a-b206d3262871"), fieldMetaData1.DataSetFieldId);
            Assert.AreEqual(0, fieldMetaData1.ArrayDimensions.Length);
            Assert.AreEqual(UADataSetFieldFlags.None, fieldMetaData1.DataSetFieldFlags);
            Assert.AreEqual("", fieldMetaData1.Description);
            Assert.AreEqual(0, fieldMetaData1.MaximumStringLength);
            Assert.AreEqual(-1, fieldMetaData1.ValueRank);

            UAFieldMetaData fieldMetaData2 = dataSetMetaData.Fields[2];
            Assert.IsNotNull(fieldMetaData2);
            Assert.AreEqual("Scalar.Int32.Y", fieldMetaData2.Name);
            Assert.AreEqual(UABuiltInType.Int32, fieldMetaData2.BuiltInType);
            Assert.AreEqual(UADataTypeIds.Int32, fieldMetaData2.DataTypeId);
            Assert.AreEqual(Guid.Parse("f29ffb7c-dfc1-4d7a-9d67-e4490b5b0fed"), fieldMetaData2.DataSetFieldId);
            Assert.AreEqual(0, fieldMetaData2.ArrayDimensions.Length);
            Assert.AreEqual(UADataSetFieldFlags.None, fieldMetaData2.DataSetFieldFlags);
            Assert.AreEqual("", fieldMetaData2.Description);
            Assert.AreEqual(0, fieldMetaData2.MaximumStringLength);
            Assert.AreEqual(-1, fieldMetaData2.ValueRank);

            UAFieldMetaData fieldMetaData3 = dataSetMetaData.Fields[3];
            Assert.IsNotNull(fieldMetaData3);
            Assert.AreEqual("DateTimeValue", fieldMetaData3.Name);
            Assert.AreEqual(UABuiltInType.DateTime, fieldMetaData3.BuiltInType);
            Assert.AreEqual(UADataTypeIds.DateTime, fieldMetaData3.DataTypeId);
            Assert.AreEqual(Guid.Parse("08dda30a-1c81-4df5-a31e-155fbcf9376f"), fieldMetaData3.DataSetFieldId);
            Assert.AreEqual(0, fieldMetaData3.ArrayDimensions.Length);
            Assert.AreEqual(UADataSetFieldFlags.None, fieldMetaData3.DataSetFieldFlags);
            Assert.AreEqual("", fieldMetaData3.Description);
            Assert.AreEqual(0, fieldMetaData3.MaximumStringLength);
            Assert.AreEqual(-1, fieldMetaData3.ValueRank);
        }

        [TestMethod]
        public void FindPubSubConnectionElementTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            UAPubSubConnectionElement pubSubConnectionElement = configuration.FindConnectionElement("UDP1");

            TestSection.Assert();
            Assert.IsNotNull(pubSubConnectionElement);
            Assert.AreEqual("UDP1", pubSubConnectionElement.Name);
            Assert.AreEqual(0, pubSubConnectionElement.CustomPropertyValueDictionary.Count);
            Assert.IsTrue(pubSubConnectionElement.Enabled);
            Assert.AreEqual("192.168.0.20", pubSubConnectionElement.NetworkAddress.InterfaceName);
            var resourceAddress = pubSubConnectionElement.NetworkAddress as ResourceAddress;
            Assert.IsNotNull(resourceAddress);
            Assert.AreEqual("opc.udp://239.0.0.1:4840", resourceAddress.ResourceDescriptor.UrlString);
            Assert.AreEqual(new UAPublisherId(UAPublisherIdType.Byte, 20), pubSubConnectionElement.PublisherId);
            Assert.IsNull(pubSubConnectionElement.TransportParameters);
            Assert.AreEqual(UAPubSubTransportProfileUriStrings.UdpUadp, pubSubConnectionElement.TransportProfileUriString);
        }

        [TestMethod]
        public void FindWriterGroupElementTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            UAWriterGroupElement writerGroupElement = configuration.FindWriterGroupElement("UDP1", "WriterGroupDynamic");

            TestSection.Assert();
            Assert.IsNotNull(writerGroupElement);
            Assert.AreEqual("WriterGroupDynamic", writerGroupElement.Name);
            Assert.AreEqual(0, writerGroupElement.CustomPropertyValueDictionary.Count);
            Assert.IsTrue(writerGroupElement.Enabled);
            Assert.AreEqual("", writerGroupElement.HeaderLayoutUriString);
            Assert.AreEqual(5000.0d, writerGroupElement.KeepAliveTime);
            Assert.AreEqual(0, writerGroupElement.LocaleIds.Count);
            var uadpWriterGroupMessageParameters = writerGroupElement.MessageParameters as UAUadpWriterGroupMessageParameters;
            Assert.IsNotNull(uadpWriterGroupMessageParameters);
            Assert.AreEqual(UADataSetOrdering.Undefined, uadpWriterGroupMessageParameters.DataSetOrdering);
            Assert.AreNotEqual(0, uadpWriterGroupMessageParameters.GroupVersion);
            Assert.IsTrue(uadpWriterGroupMessageParameters.GroupVersionDateTime.Year >= 2019);
            Assert.AreEqual(UAUadpNetworkMessageContentMask.PublisherId | UAUadpNetworkMessageContentMask.PayloadHeader, uadpWriterGroupMessageParameters.NetworkMessageContentMask);
            Assert.AreEqual(0.0d, uadpWriterGroupMessageParameters.PublishingOffset);
            Assert.AreEqual(1, uadpWriterGroupMessageParameters.PublishingOffsets.Length);
            Assert.AreEqual(0.0d, uadpWriterGroupMessageParameters.SamplingOffset);
            Assert.AreEqual(0, writerGroupElement.Priority);
            Assert.AreEqual(100.0d, writerGroupElement.PublishingInterval);
            var datagramWriterGroupTransportParameters = writerGroupElement.TransportParameters as UADatagramWriterGroupTransportParameters;
            Assert.IsNotNull(datagramWriterGroupTransportParameters);
            Assert.AreEqual(0, datagramWriterGroupTransportParameters.MessageRepeatCount);
            Assert.AreEqual(0.0d, datagramWriterGroupTransportParameters.MessageRepeatDelay);
            Assert.AreEqual(2, writerGroupElement.WriterGroupId);
        }

        [TestMethod]
        public void GetDataSetFolderNamesTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            StringCollection dataSetFolderNames = configuration.ListDataSetFolderNames(new string[0]);

            TestSection.Assert();
            Assert.IsNotNull(dataSetFolderNames);
            Assert.AreEqual(0, dataSetFolderNames.Count);
        }

        [TestMethod]
        public void GetDataSetSourceElementTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            UAPublishedDataSetSourceElement publishedDataSetSourceElement = configuration.GetPublishedDataSetSourceElement("Simple");

            TestSection.Assert();
            Assert.IsNotNull(publishedDataSetSourceElement);
            var publishedDataItemsElement = publishedDataSetSourceElement as UAPublishedDataItemsElement;
            Assert.IsNotNull(publishedDataItemsElement);
            Assert.AreEqual(4, publishedDataItemsElement.PublishedVariables.Count);
            // IMPROVE: assert more
        }

        [TestMethod]
        public void GetDataSetWriterNamesTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            StringCollection dataSetWriterNames = configuration.ListDataSetWriterNames("UDP1", "WriterGroupStatic");

            TestSection.Assert();
            Assert.IsNotNull(dataSetWriterNames);
            Assert.AreEqual(3, dataSetWriterNames.Count);
            Assert.AreEqual("WriterSimple", dataSetWriterNames[0]);
            Assert.AreEqual("WriterAllTypes", dataSetWriterNames[1]);
            Assert.AreEqual("WriterMassData", dataSetWriterNames[2]);
        }

        [TestMethod]
        public void GetExtensionFieldQualifiedNamesTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            UAQualifiedNameSet extensionFieldQualifiedNames = configuration.ListExtensionFieldQualifiedNames("Simple");

            TestSection.Assert();
            Assert.IsNotNull(extensionFieldQualifiedNames);
            Assert.AreEqual(0, extensionFieldQualifiedNames.Count);
        }

        [TestMethod]
        public void GetPublishedDataNamesTest_NonRecursive()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            StringCollection publishedDataSetNames = configuration.ListPublishedDataSetNames(new string[0], recursive:false);

            TestSection.Assert();
            Assert.IsNotNull(publishedDataSetNames);
            Assert.AreEqual(3, publishedDataSetNames.Count);
            Assert.AreEqual("Simple", publishedDataSetNames[0]);
            Assert.AreEqual("AllTypes", publishedDataSetNames[1]);
            Assert.AreEqual("MassData", publishedDataSetNames[2]);
        }

        [TestMethod]
        public void GetPublishedDataNamesTest_Recursive()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            StringCollection publishedDataSetNames = configuration.ListPublishedDataSetNames(new string[0], recursive: true);

            TestSection.Assert();
            Assert.IsNotNull(publishedDataSetNames);
            Assert.AreEqual(3, publishedDataSetNames.Count);
            Assert.AreEqual("Simple", publishedDataSetNames[0]);
            Assert.AreEqual("AllTypes", publishedDataSetNames[1]);
            Assert.AreEqual("MassData", publishedDataSetNames[2]);
        }

        [TestMethod]
        public void GetPubSubConfigurationElementTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            UAPubSubConfigurationElement pubSubConfigurationElement = configuration.GetPubSubConfigurationElement();

            TestSection.Assert();
            Assert.IsNotNull(pubSubConfigurationElement);
            Assert.IsTrue(pubSubConfigurationElement.Enabled);
        }

        [TestMethod]
        public void GetPubSubConnectionNamesTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            StringCollection pubSubConnectionNames = configuration.ListConnectionNames();

            TestSection.Assert();
            Assert.IsNotNull(pubSubConnectionNames);
            Assert.AreEqual(1, pubSubConnectionNames.Count);
            Assert.AreEqual("UDP1", pubSubConnectionNames[0]);
        }

        //[TestMethod]
        //public void GetReaderGroupNamesTest()
        //{
        //    TestSection.Arrange();
        //    var configuration = CreateConfiguration();

        //    TestSection.Act();
        //    StringCollection readerGroupNames = configuration.GetReaderGroupNames("UDP1");

        //    TestSection.Assert();
        //    Assert.IsNotNull(readerGroupNames);
        //    Assert.AreEqual(0, readerGroupNames.Count);
        //}

        [TestMethod]
        public void GetWriterGroupNamesTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            StringCollection writerGroupNames = configuration.ListWriterGroupNames("UDP1");

            TestSection.Assert();
            Assert.IsNotNull(writerGroupNames);
            Assert.AreEqual(2, writerGroupNames.Count);
            Assert.AreEqual("WriterGroupStatic", writerGroupNames[0]);
            Assert.AreEqual("WriterGroupDynamic", writerGroupNames[1]);
        }

        [TestMethod]
        public void HasDataSetFolderTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            bool hasDataSetFolder = configuration.HasDataSetFolder(new string[0], "SomeName");

            TestSection.Assert();
            Assert.IsFalse(hasDataSetFolder);
        }

        [TestMethod]
        public void HasExtensionFieldTest()
        {
            TestSection.Arrange();
            var configuration = CreateConfiguration();

            TestSection.Act();
            bool result = configuration.HasExtensionField("Simple", UAExtensionFields.DataSetName);

            TestSection.Assert();
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void LoadConfigurationFromFileNodeTest()
        {
            TestSection.Arrange();
            UAEndpointDescriptor endpointDescriptor =
                UALiveTestEnvironment.UnifiedAutomation_UaCppServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);
            var publishSubscribeClient = new EasyUAPublishSubscribeClient();
            
            TestSection.Act();
            IUAReadOnlyPubSubConfiguration readOnlyPubSubConfiguration = 
                publishSubscribeClient.LoadReadOnlyConfigurationFromFileNode(endpointDescriptor);
            Assert.IsNotNull(readOnlyPubSubConfiguration);

            // IMPROVE: assert more
        }

        #endregion



        #region Private Properties

        [NotNull]
        private UALiveTestEnvironment TestEnvironment => UALiveTestEnvironment.Current;

        #endregion

        
        
        #region Private Methods

        [NotNull]
        private UAPubSubClientConfiguration CreateConfiguration()
        {
            UAEndpointDescriptor endpointDescriptor =
                UALiveTestEnvironment.UnifiedAutomation_UaCppServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);
            AssertUtilities.WriteLine($"Creating client configuration for endpoint: {endpointDescriptor}");
            var configuration = new UAPubSubClientConfiguration(new EasyUAPublishSubscribeClient(), endpointDescriptor);
            return configuration;
        }

        #endregion
    }
}
