﻿
using Microsoft.Extensions.DependencyInjection;
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Browsing;
using OpcLabs.BaseLib.Connectivity.Point;
using OpcLabs.BaseLib.Extensions;
using OpcLabs.EasyOpc.UA.Connectivity;
using OpcLabs.EasyOpc.UA.Connectivity.Implementation;

namespace OpcLabs.EasyOpcTest.UA.Connectivity
{
    public partial class UAConnectivityTest
    {
        #region Static Private Methods

        static private void GetServiceTest_Failure([NotNull] Type serviceType)
        {
            var connectivity = new UAConnectivity();
            object service = connectivity.GetService(serviceType);
            Assert.IsNull(service);
        }

        static private void GetServiceTest_Success([NotNull] Type serviceType)
        {
            var connectivity = new UAConnectivity();
            object service = connectivity.GetService(serviceType);
            Assert.IsNotNull(service);
            Assert.IsInstanceOfType(service, serviceType);
        }

        #endregion



        #region Test Methods

        [TestCategory("OpcUA")]
        [TestMethod]
        public void CreatePointBrowseableTest()
        {
            var connectivity = new UAConnectivity();
            // ReSharper disable once PossibleNullReferenceException
            IBrowseable browseable = connectivity.GetService<IBrowsePointService>().CreatePointBrowseable(new object());
            Assert.IsNotNull(browseable);
            // IMPROVE: test more
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void CreateReaderTest()
        {
            var connectivity = new UAConnectivity();
            // ReSharper disable once PossibleNullReferenceException
            var pointReader = (UAPointReader) connectivity.GetService<IReadPointService>().CreateReader();
            Assert.IsNotNull(pointReader);
            Assert.IsFalse(pointReader.IsFrozen);
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void CreateSubscriberTest()
        {
            var connectivity = new UAConnectivity();
            // ReSharper disable once PossibleNullReferenceException
            var pointSubscriber = (UAPointSubscriber) connectivity.GetService<ISubscribePointService>().CreateSubscriber();
            Assert.IsNotNull(pointSubscriber);
            Assert.IsFalse(pointSubscriber.IsFrozen);
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void CreateWriterTest()
        {
            var connectivity = new UAConnectivity();
            // ReSharper disable once PossibleNullReferenceException
            var pointWriter = (UAPointWriter) connectivity.GetService<IWritePointService>().CreateWriter();
            Assert.IsNotNull(pointWriter);
            Assert.IsFalse(pointWriter.IsFrozen);
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void GetServiceTest_ICreateParametersService()
        {
            GetServiceTest_Success(typeof(ICreateParametersService));
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void GetServiceTest_ICreatePointService()
        {
            GetServiceTest_Success(typeof(ICreatePointService));
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void GetServiceTest_IBrowsePointService()
        {
            GetServiceTest_Success(typeof(IBrowsePointService));
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void GetServiceTest_IReadPointService()
        {
            GetServiceTest_Success(typeof(IReadPointService));
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void GetServiceTest_ISubscribePointService()
        {
            GetServiceTest_Success(typeof(ISubscribePointService));
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void GetServiceTest_IWritePointService()
        {
            GetServiceTest_Success(typeof(IWritePointService));
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void GetServiceTest_Object()
        {
            GetServiceTest_Failure(typeof(Object));
        }

        #endregion
    }
}
