﻿
using System;
using System.Diagnostics;
using System.IO;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.IO.Extensions;
using OpcLabs.BaseLib.Utilities;

namespace OpcLabs.EasyOpcTest.UA.DataTypeModel.Internal
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    public /*partial*/ class OpcFoundation_SampleServerTest : UAOpcBinaryDataTypeSystemTest
    {
        #region Static Private Properties

        [NotNull]
        // ReSharper disable once InconsistentNaming
        static private byte[] StructureExtensionObjectBody
        {
            get
            {
                var bodyHexaString = 
                    /*BooleanValue*/            "01-" +
                    /*SByteValue*/              "9E-" +
                    /*ByteValue*/               "34-" +
                    /*Int16Value*/              "D0-31-" +
                    /*UInt16Value*/             "0E-5F-" +
                    /*Int32Value*/              "78-1D-26-45-" +
                    /*UInt32Value*/             "06-B0-70-93-" +
                    /*Int64Value*/              "EB-FA-D1-E0-B9-63-EF-B2-" +
                    /*UInt64Value*/             "1B-A2-2A-20-AE-E8-25-4E-" +
                    /*FloatValue*/              "4D-64-41-D1-" +
                    /*DoubleValue*/             "00-00-00-20-FE-94-A5-BA-" +
                    /*StringValue should be String, but is incorrectly encoded as CharArray*/
                    /*StringValue*/             "1C-00-00-00-E3-83-91-E3-82-A4-E3-83-8A-E3-83-83-E3-83-97-E3-83-AB-20-E9-BB-92-2C-20-E7-8C-BF-3C-" +
                    /*DateTimeValue*/           "A1-3D-42-E6-B4-D1-A3-01-" +
                    /*GuidValue*/               "F6-8B-73-6F-B0-03-CF-2B-A1-6F-9E-A1-44-CE-3C-A6-" +
                    /*ByteStringValue*/         "0D-00-00-00-2C-C6-8C-2F-DE-53-61-E5-25-00-E5-C4-C0-" +
                    /*XmlElementValue*/         "EC-05-00-00-3C-6E-30-3A-ED-8C-8C-EC-9D-B8-EC-95-A0-ED-94-8C-20-EA-B2-80-EC-A0-95-3D-22-D0-98-D0-B7-D0-B2-D0-B5-D1-81-D1-82-D0-BA-D0-B0-22-20-EB-B3-B5-EC-88-AD-EC-95-84-3D-22-D0-9B-D0-B8-D0-BC-D0-BE-D0-BD-22-20-EA-B0-9C-3D-22-D0-9B-D0-B8-D0-BC-D0-BE-D0-BD-22-20-EB-93-A4-EC-AD-89-ED-8C-8C-EC-9D-B8-EC-95-A0-ED-94-8C-3D-22-D0-A1-D0-B2-D0-B8-D0-BD-D1-8C-D1-8F-22-20-EB-A0-88-EB-AA-AC-3D-22-D0-9B-D0-BE-D1-88-D0-B0-D0-B4-D1-8C-22-20-78-6D-6C-6E-73-3A-6E-30-3D-22-68-74-74-70-3A-2F-2F-EB-8F-BC-EC-A7-80-22-3E-3C-6E-30-3A-EB-93-A4-EC-AD-89-3E-D0-9A-D1-80-D0-B0-D1-81-D0-BD-D0-BE-20-D0-A1-D0-B2-D0-B8-D0-BD-D1-8C-D1-8F-5D-20-D0-90-D0-BD-D0-B0-D0-BD-D0-B0-D1-81-26-6C-74-3B-20-D0-96-D0-B5-D0-BB-D1-82-D1-8B-D0-B9-D1-86-D0-B2-D0-B5-D1-82-2C-20-D0-9A-D0-BE-D1-82-2A-20-D0-98-D0-B7-D0-B2-D0-B5-D1-81-D1-82-D0-BA-D0-B0-20-D0-98-D0-B7-D0-B2-D0-B5-D1-81-D1-82-D0-BA-D0-B0-20-D0-9A-D1-80-D0-B0-D1-81-D0-BD-D0-BE-2A-3C-2F-6E-30-3A-EB-93-A4-EC-AD-89-3E-3C-6E-30-3A-EB-B3-B5-EC-88-AD-EC-95-84-3E-D0-94-D1-80-D0-B0-D0-BA-D0-BE-D0-BD-20-D0-9F-D1-83-D1-80-D0-BF-D1-83-D1-80-D0-BE-D0-B2-D0-BE-20-D0-9B-D0-B8-D0-BC-D0-BE-D0-BD-29-20-D0-9F-D0-B5-D1-80-D1-81-D0-B8-D0-BA-7D-20-D0-9B-D0-BE-D1-88-D0-B0-D0-B4-D1-8C-25-20-D0-A7-D0-B5-D1-80-D0-BD-D0-BE-D1-82-D0-B0-3D-3C-2F-6E-30-3A-EB-B3-B5-EC-88-AD-EC-95-84-3E-3C-6E-30-3A-EB-B3-B5-EC-88-AD-EC-95-84-3E-D0-96-D0-B5-D0-BB-D1-82-D1-8B-D0-B9-D1-86-D0-B2-D0-B5-D1-82-20-D1-8F-D0-B1-D0-BB-D0-BE-D0-BA-D0-BE-5E-20-D1-8F-D0-B1-D0-BB-D0-BE-D0-BA-D0-BE-20-D0-9F-D1-83-D1-80-D0-BF-D1-83-D1-80-D0-BE-D0-B2-D0-BE-26-6C-74-3B-20-D0-93-D0-BE-D0-BB-D1-83-D0-B1-D0-B8-D0-BA-D0-B0-20-D0-A1-D0-BE-D0-B1-D0-B0-D0-BA-D0-B0-3C-2F-6E-30-3A-EB-B3-B5-EC-88-AD-EC-95-84-3E-3C-6E-30-3A-EB-8F-BC-EC-A7-80-3E-D0-91-D0-B5-D0-BB-D0-BE-22-20-D0-A1-D0-B2-D0-B8-D0-BD-D1-8C-D1-8F-20-D0-A1-D0-BE-D0-B1-D0-B0-D0-BA-D0-B0-20-D0-97-D0-B5-D0-BB-D0-B5-D0-BD-D1-8B-D0-B9-D1-86-D0-B2-D0-B5-D1-82-5E-20-D1-8F-D0-B1-D0-BB-D0-BE-D0-BA-D0-BE-3B-20-D0-9E-D0-B2-D1-86-D1-8B-20-D1-8F-D0-B1-D0-BB-D0-BE-D0-BA-D0-BE-20-D0-9E-D0-B2-D1-86-D1-8B-3C-2F-6E-30-3A-EB-8F-BC-EC-A7-80-3E-3C-6E-30-3A-ED-8C-8C-EB-9E-80-EB-A7-9D-EA-B3-A0-3E-D0-A1-D0-BB-D0-BE-D0-BD-20-D0-9B-D0-BE-D1-88-D0-B0-D0-B4-D1-8C-20-D0-9F-D0-B5-D1-80-D1-81-D0-B8-D0-BA-20-D0-92-D0-B8-D0-BD-D0-BE-D0-B3-D1-80-D0-B0-D0-B4-D0-B8-D0-BD-D0-B0-2F-3C-2F-6E-30-3A-ED-8C-8C-EB-9E-80-EB-A7-9D-EA-B3-A0-3E-3C-6E-30-3A-EC-9A-A9-EC-9A-A9-EB-94-B8-EA-B8-B0-3E-D0-9A-D0-BE-D1-80-D0-BE-D0-B2-D0-B0-20-D0-91-D0-B5-D0-BB-D0-BE-26-61-6D-70-3B-20-D0-A1-D0-BE-D0-B1-D0-B0-D0-BA-D0-B0-24-20-D0-9E-D0-B1-D0-B5-D0-B7-D1-8C-D1-8F-D0-BD-D0-B0-60-20-D0-97-D0-BC-D0-B5-D0-B9-D0-BA-D0-B0-20-D0-97-D0-B5-D0-BB-D0-B5-D0-BD-D1-8B-D0-B9-D1-86-D0-B2-D0-B5-D1-82-20-D0-92-D0-B8-D0-BD-D0-BE-D0-B3-D1-80-D0-B0-D0-B4-D0-B8-D0-BD-D0-B0-23-3C-2F-6E-30-3A-EC-9A-A9-EC-9A-A9-EB-94-B8-EA-B8-B0-3E-3C-6E-30-3A-EB-B9-A8-EA-B0-84-3E-D0-A7-D0-B5-D1-80-D0-BD-D0-BE-D1-82-D0-B0-26-61-6D-70-3B-20-D0-9A-D0-BE-D1-80-D0-BE-D0-B2-D0-B0-20-D0-91-D0-B5-D0-BB-D0-BE-20-D0-92-D0-B8-D0-BD-D0-BE-D0-B3-D1-80-D0-B0-D0-B4-D0-B8-D0-BD-D0-B0-3B-20-D0-97-D0-BC-D0-B5-D0-B9-D0-BA-D0-B0-20-D0-97-D0-B5-D0-BB-D0-B5-D0-BD-D1-8B-D0-B9-D1-86-D0-B2-D0-B5-D1-82-20-D0-9A-D1-80-D1-8B-D1-81-D0-B0-60-20-D0-91-D0-B0-D0-BD-D0-B0-D0-BD-2B-20-D0-9A-D1-80-D1-8B-D1-81-D0-B0-20-D0-97-D0-BC-D0-B5-D0-B9-D0-BA-D0-B0-3C-2F-6E-30-3A-EB-B9-A8-EA-B0-84-3E-3C-6E-30-3A-EB-94-B8-EA-B8-B0-3E-D0-A1-D0-B2-D0-B8-D0-BD-D1-8C-D1-8F-21-20-D0-9E-D0-B2-D1-86-D1-8B-5D-20-D0-97-D0-BC-D0-B5-D0-B9-D0-BA-D0-B0-20-D0-93-D0-BE-D0-BB-D1-83-D0-B1-D0-BE-20-D0-98-D0-B7-D0-B2-D0-B5-D1-81-D1-82-D0-BA-D0-B0-20-D0-90-D0-BD-D0-B0-D0-BD-D0-B0-D1-81-20-D0-90-D0-BD-D0-B0-D0-BD-D0-B0-D1-81-20-D0-92-D0-B8-D0-BD-D0-BE-D0-B3-D1-80-D0-B0-D0-B4-D0-B8-D0-BD-D0-B0-20-D0-A7-D0-B5-D1-80-D0-BD-D0-BE-D1-82-D0-B0-2F-3C-2F-6E-30-3A-EB-94-B8-EA-B8-B0-3E-3C-6E-30-3A-ED-8C-8C-EB-9E-80-3E-D0-92-D0-B8-D0-BD-D0-BE-D0-B3-D1-80-D0-B0-D0-B4-D0-B8-D0-BD-D0-B0-26-67-74-3B-20-D0-94-D1-80-D0-B0-D0-BA-D0-BE-D0-BD-20-D0-9A-D0-BE-D1-80-D0-BE-D0-B2-D0-B0-29-20-D0-91-D0-B0-D0-BD-D0-B0-D0-BD-5D-20-D0-90-D0-BD-D0-B0-D0-BD-D0-B0-D1-81-2F-20-D0-90-D0-BD-D0-B0-D0-BD-D0-B0-D1-81-7D-20-D0-90-D0-BD-D0-B0-D0-BD-D0-B0-D1-81-20-D0-93-D0-BE-D0-BB-D1-83-D0-B1-D0-BE-2C-20-D0-91-D0-B5-D0-BB-D0-BE-20-D0-A1-D0-BE-D0-B1-D0-B0-D0-BA-D0-B0-3C-2F-6E-30-3A-ED-8C-8C-EB-9E-80-3E-3C-6E-30-3A-EB-A0-88-EB-AA-AC-EC-BD-94-EB-81-BC-EB-A6-AC-3E-D0-9A-D0-BE-D1-82-20-D0-91-D0-B5-D0-BB-D0-BE-20-D0-9A-D1-80-D1-8B-D1-81-D0-B0-20-D0-9A-D0-BE-D1-80-D0-BE-D0-B2-D0-B0-21-3C-2F-6E-30-3A-EB-A0-88-EB-AA-AC-EC-BD-94-EB-81-BC-EB-A6-AC-3E-3C-2F-6E-30-3A-ED-8C-8C-EC-9D-B8-EC-95-A0-ED-94-8C-3E-" +
                    /*NodeIdValue*/             "03-05-00-0A-00-00-00-D0-9B-D0-B8-D0-BC-D0-BE-D0-BD-" +
                    /*ExpandedNodeIdValue*/     "82-00-00-37-95-E2-9F-22-00-00-00-68-74-74-70-3A-2F-2F-73-61-6D-70-6C-65-73-2E-6F-72-67-2F-55-41-2F-6D-65-6D-6F-72-79-62-75-66-66-65-72-" +
                    /*QualifiedNameValue has incorrectly a UInt16 namespace index while it should be Int32*/
                    /*QualifiedNameValue*/      "07-00-05-00-00-00-53-6E-61-6B-65-" +
                    /*LocalizedTextValue*/      "03-05-00-00-00-65-6E-2D-55-53-05-00-00-00-53-68-65-65-70-" +
                    /*StatusCodeValue*/         "00-00-05-00-" +
                    /*VariantValue*/            "11-02-03-00-6A-CC-21-A0-" +
                    /*EnumerationValue*/        "00-00-00-00-" +
                    /*StructureValue has incorrectly a NodeId at the first place (instead of ExpandedNodeId at the second place), and is missing the body at the end*/
                    /*StructureValue*/          "00-00-00-" +
                    /*Number*/                  "0B-00-00-00-00-00-00-00-00-" + 
                    /*Integer*/                 "08-00-00-00-00-00-00-00-00-" +
                    /*UInteger*/                "09-00-00-00-00-00-00-00-00";
                byte[] extensionObjectBody = BitConverterUtilities.ArrayOfBytesFromString(bodyHexaString);
                return extensionObjectBody;
            }
        }

        [NotNull]
        static private byte[] TypeDictionaryByteArray
        {
            get
            {
                Type type = typeof(UAOpcBinaryDataTypeSystemTest);
                Stream stream = type.Assembly.GetManifestResourceStream(type, "OpcFoundation_SampleServer_OpcBinaryTypeDictionary.xml");
                Debug.Assert(stream != null);
                byte[] dictionaryByteArray = stream.ReadToArrayOfBytes();
                return dictionaryByteArray;
            }
        }

        #endregion



        #region Test Methods

        [TestMethod]
        public void DecodeTest_Structure() =>
            DecodeTest(null,
                TypeDictionaryByteArray,
                "ScalarValueDataType",
                StructureExtensionObjectBody);  // IMPROVE: check at least the returned data type, if not the value as well

        #endregion
    }
}
