﻿
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.EasyOpc.UA.DataTypeModel;

namespace OpcLabs.EasyOpcTest.UA.DataTypeModel.Internal
{
    partial class TypeDescriptionExamplesTest
    {
        #region Static Private Properties

        [NotNull]
        static private StructuredDataType IntegerArray2DataType
        {
            get
            {
                var integerArray2DataType = new StructuredDataType("IntegerArray2", "TypeDescriptionExamples:IntegerArray2")
                {
                    new DataField("Value", new SequenceDataType(UAOpcBinaryStandardDataTypes.Int16))
                };
                integerArray2DataType.Description = "An array of integers terminated with a known value.";
                return integerArray2DataType;
            }
        }

        [NotNull]
        static private StructuredData IntegerArray2_0Data => new StructuredData(IntegerArray2DataType)
        {
            {"Value", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int16)) }
        };

        [NotNull]
        static private byte[] IntegerArray2_0ExtensionObjectBody => new byte[] { 0xFF, 0x7F };

        [NotNull]
        static private StructuredData IntegerArray2_1Data => new StructuredData(IntegerArray2DataType)
        {
            {
                "Value", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int16))
                {
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int16, (Int16)11)
                }
            }
        };

        [NotNull]
        static private byte[] IntegerArray2_1ExtensionObjectBody => new byte[] { 11, 0, 0xFF, 0x7F };

        [NotNull]
        static private StructuredData IntegerArray2_3Data => new StructuredData(IntegerArray2DataType)
        {
            {
                "Value", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int16))
                {
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int16, (Int16)11),
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int16, (Int16)22),
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int16, (Int16)33)
                }
            }
        };

        [NotNull]
        static private byte[] IntegerArray2_3ExtensionObjectBody => new byte[] { 11, 0, 22, 0, 33, 0, 0xFF, 0x7F };

        #endregion



        #region Test Methods

        [TestMethod]
        public void DecodeTest_IntegerArray2_0() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray2", IntegerArray2_0ExtensionObjectBody, IntegerArray2_0Data);

        [TestMethod]
        public void DecodeTest_IntegerArray2_1() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray2", IntegerArray2_1ExtensionObjectBody, IntegerArray2_1Data);

        [TestMethod]
        public void DecodeTest_IntegerArray2_3() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray2", IntegerArray2_3ExtensionObjectBody, IntegerArray2_3Data);

        [TestMethod]
        public void EncodeTest_IntegerArray2_0() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray2", IntegerArray2_0Data, IntegerArray2_0ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_IntegerArray2_1() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray2", IntegerArray2_1Data, IntegerArray2_1ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_IntegerArray2_3() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray2", IntegerArray2_3Data, IntegerArray2_3ExtensionObjectBody);

        #endregion
    }
}
