﻿
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Graphs;
using OpcLabs.BaseLib.Graphs.Extensions;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.Extensions;
using OpcLabs.EasyOpc.UA.Graphs;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA
{
    partial class UAInteroperabilityWorkshopTest
    {
        #region Test Methods

        [TestMethod]
        public void Group011_AlarmCondition_Header____________________()
        {
        }

        /// <summary>
        /// Browse() the Types folder for supported Alarm types and Condition types.
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test01_Browse_Types()
        {
            TestSection.Arrange();
            const int count = 100;
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient { Isolated = true })
            {
                UABrowseParameters browseParameters = UABrowseParameters.ObjectTypes;
                IEnumerable<UANodeTreePosition> treePositions = client.BrowseTree(endpointDescriptor, UAObjectTypeIds.BaseEventType, browseParameters).ToEnumerable(TreeTraversalType.BreadthFirst);
                int i = 0;
                foreach (UANodeTreePosition treePosition in treePositions.Take(count))
                {
                    // ReSharper disable once PossibleNullReferenceException
                    AssertUtilities.WriteLine($"[{i}] ({treePosition.Level}) {treePosition.DisplayName}: {treePosition}");
                    i++;
                }
                Assert.IsTrue(i >= 1, "Not enough elements");
            }
        }

        /// <summary>
        /// Call CreateMonitoredItems() for an Event Notifier. Specify event fields to be returned. Specify a simple filter (e.g. if (Severity > 400)). Vary the selection and the filter.
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test02_CreateMonitoredItems_EventNotifier()
        {
            ClientNotSupported();   // improve: have this
        }

        /// <summary>
        /// [obsolete] Publish service delivers Alarm and Condition events
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test03_Publish_AlarmAndCondition()
        {
            ClientNotSupported();   // improve: have this
        }

        /// <summary>
        /// Call ConditionRefresh for a subscription while at least one Condition is in an interesting state (e.g. an Alarm is active). Verify that this Condition state is reported.
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test04_ConditionRefresh()
        {
            ClientNotSupported();   // improve: have this
        }

        /// <summary>
        /// Modify the state of a Condition by calling the Enabl/Disable methods. Verify the reporting behaviour.
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test05_Condition_EnableDisable()
        {
            ClientNotSupported();   // improve: have this
        }

        /// <summary>
        /// Call the Acknowledge Method for an Alarm that requires acknowledgement.
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test06_Acknowledge()
        {
            ClientNotSupported();   // improve: have this
        }

        /// <summary>
        /// Call the Confirm Method for an Alarm that requires confirmation.
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test07_Confirm()
        {
            ClientNotSupported();   // improve: have this
        }

        /// <summary>
        /// [obsolete] Dialog Condition including Respond Method
        /// </summary>
        [TestMethod]
        public void Group011_AlarmCondition_Test0_DialogCondition_Respond()
        {
            ClientNotSupported();   // improve: have this
        }

        #endregion
    }
}
